@@ -7,7 +7,9 @@
  */
 
 
-#ifndef WIN32
+#ifdef WIN32
+#include "win32config.h"
+#else
 #include "config.h"
 #endif
 #include <stdio.h>
@@ -26,6 +28,9 @@
 #include "entities.h"
 #include "valid.h"
 
+static void
+htmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur);
+
 /**
  * htmlDtdDump:
  * @buf:  the HTML buffer output
@@ -106,7 +111,7 @@ htmlAttrListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
 }
 
 
-static void
+void
 htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur);
 /**
  * htmlNodeListDump:
@@ -136,7 +141,7 @@ htmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
  *
  * Dump an HTML node, recursive behaviour,children are printed too.
  */
-static void
+void
 htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
     htmlElemDescPtr info;
 
@@ -147,6 +152,10 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
     /*
      * Special cases.
      */
+    if (cur->type == XML_HTML_DOCUMENT_NODE) {
+	htmlDocContentDump(buf, (xmlDocPtr) cur);
+	return;
+    }
     if (cur->type == HTML_TEXT_NODE) {
 	if (cur->content != NULL) {
             xmlChar *buffer;
@@ -301,7 +310,7 @@ htmlDocDumpMemory(xmlDocPtr cur, xmlChar**mem, int *size) {
 
     if (cur == NULL) {
 #ifdef DEBUG_TREE
-        fprintf(stderr, "xmlDocDumpMemory : document == NULL\n");
+        fprintf(stderr, "htmlxmlDocDumpMemory : document == NULL\n");
 #endif
 	*mem = NULL;
 	*size = 0;
@@ -334,7 +343,7 @@ htmlDocDump(FILE *f, xmlDocPtr cur) {
 
     if (cur == NULL) {
 #ifdef DEBUG_TREE
-        fprintf(stderr, "xmlDocDump : document == NULL\n");
+        fprintf(stderr, "htmlDocDump : document == NULL\n");
 #endif
 	return;
     }
