@@ -7,8 +7,7 @@
  */
 
 #ifdef WIN32
-#define HAVE_FCNTL_H
-#include <io.h>
+#include "win32config.h"
 #else
 #include "config.h"
 #endif
@@ -31,10 +30,19 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_LIBREADLINE
+#include <readline/readline.h>
+#ifdef HAVE_LIBHISTORY
+#include <readline/history.h>
+#endif
+#endif
 
 #include "xmlmemory.h"
 #include "parser.h"
+#include "HTMLparser.h"
+#include "HTMLtree.h"
 #include "tree.h"
+#include "xpath.h"
 #include "debugXML.h"
 
 static int debug = 0;
@@ -48,20 +56,89 @@ static int postvalid = 0;
 static int repeat = 0;
 static int insert = 0;
 static int compress = 0;
+static int html = 0;
+static int shell = 0;
+static int push = 0;
 
 extern int xmlDoValidityCheckingDefaultValue;
 
+/**
+ * xmlShellReadline:
+ * @prompt:  the prompt value
+ *
+ * Read a string
+ * 
+ * Returns a pointer to it or NULL on EOF the caller is expected to
+ *     free the returned string.
+ */
+char *
+xmlShellReadline(char *prompt) {
+#ifdef HAVE_LIBREADLINE
+    char *line_read;
+
+    /* Get a line from the user. */
+    line_read = readline (prompt);
+
+    /* If the line has any text in it, save it on the history. */
+    if (line_read && *line_read)
+	add_history (line_read);
+
+    return (line_read);
+#else
+    char line_read[501];
+
+    if (prompt != NULL)
+	fprintf(stdout, "%s", prompt);
+    if (!fgets(line_read, 500, stdin))
+        return(NULL);
+    line_read[500] = 0;
+    return(strdup(line_read));
+#endif
+}
 
 void parseAndPrintFile(char *filename) {
-    xmlDocPtr doc, tmp;
+    xmlDocPtr doc = NULL, tmp;
+
+    if (html) {
+	doc = htmlParseFile(filename, NULL);
+    } else {
+	/*
+	 * build an XML tree from a string;
+	 */
+	if (push) {
+	    FILE *f;
+
+	    f = fopen(filename, "r");
+	    if (f != NULL) {
+	        int res, size = 3;
+	        char chars[1024];
+                xmlParserCtxtPtr ctxt;
+
+		if (repeat)
+		    size = 1024;
+		res = fread(chars, 1, 4, f);
+		if (res > 0) {
+		    ctxt = xmlCreatePushParserCtxt(NULL, NULL,
+		                chars, res, filename);
+		    while ((res = fread(chars, 1, size, f)) > 0) {
+			xmlParseChunk(ctxt, chars, res, 0);
+		    }
+		    xmlParseChunk(ctxt, chars, 0, 1);
+		    doc = ctxt->myDoc;
+		    xmlFreeParserCtxt(ctxt);
+	        }
+	    }
+	} else if (recovery)
+	    doc = xmlRecoverFile(filename);
+	else
+	    doc = xmlParseFile(filename);
+    }
 
     /*
-     * build an XML tree from a string;
+     * shell interraction
      */
-    if (recovery)
-	doc = xmlRecoverFile(filename);
-    else
-	doc = xmlParseFile(filename);
+    if (shell)  
+        xmlShell(doc, filename, xmlShellReadline, stdout);
 
     /*
      * test intermediate copy if needed.
@@ -72,7 +149,7 @@ void parseAndPrintFile(char *filename) {
 	xmlFreeDoc(tmp);
     }
 
-    if (insert) {
+    if ((insert) && (!html)) {
         const xmlChar* list[256];
 	int nb, i;
 	xmlNodePtr node;
@@ -117,7 +194,7 @@ void parseAndPrintFile(char *filename) {
 	xmlValidateDocument(&cvp, doc);
     }
 
-    if (debugent)	
+    if ((debugent) && (!html))
 	xmlDebugDumpEntities(stdout, doc);
 
     /*
@@ -158,11 +235,23 @@ int main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-repeat")) ||
 	         (!strcmp(argv[i], "--repeat")))
 	    repeat++;
+	else if ((!strcmp(argv[i], "-push")) ||
+	         (!strcmp(argv[i], "--push")))
+	    push++;
 	else if ((!strcmp(argv[i], "-compress")) ||
 	         (!strcmp(argv[i], "--compress"))) {
 	    compress++;
 	    xmlSetCompressMode(9);
         }
+	else if ((!strcmp(argv[i], "-html")) ||
+	         (!strcmp(argv[i], "--html"))) {
+	    html++;
+        }
+	else if ((!strcmp(argv[i], "-shell")) ||
+	         (!strcmp(argv[i], "--shell"))) {
+	    shell++;
+            noout = 1;
+        }
     }
     if (noent != 0) xmlSubstituteEntitiesDefault(1);
     if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
@@ -191,6 +280,9 @@ int main(int argc, char **argv) {
 	printf("\t--repeat : repeat 100 times, for timing or profiling\n");
 	printf("\t--insert : ad-hoc test for valid insertions\n");
 	printf("\t--compress : turn on gzip compression of output\n");
+	printf("\t--html : use the HTML parser\n");
+	printf("\t--shell : run a navigating shell\n");
+	printf("\t--push : use the push mode of the parser\n");
     }
     xmlCleanupParser();
     xmlMemoryDump();
