@@ -18,7 +18,9 @@
 extern "C" {
 #endif
 
-typedef struct xmlParserInputBuffer {
+typedef struct _xmlParserInputBuffer xmlParserInputBuffer;
+typedef xmlParserInputBuffer *xmlParserInputBufferPtr;
+struct _xmlParserInputBuffer {
     /* Inputs */
     FILE          *file;    /* Input on file handler */
     void*        gzfile;    /* Input on a compressed stream */
@@ -29,14 +31,16 @@ typedef struct xmlParserInputBuffer {
     
     xmlBufferPtr buffer;    /* Local buffer encoded in  UTF-8 */
 
-} xmlParserInputBuffer;
+};
 
-typedef xmlParserInputBuffer *xmlParserInputBufferPtr;
 
 /*
  * Interfaces
  */
 
+xmlParserInputBufferPtr
+	xmlAllocParserInputBuffer		(xmlCharEncoding enc);
+
 xmlParserInputBufferPtr
 	xmlParserInputBufferCreateFilename	(const char *filename,
                                                  xmlCharEncoding enc);
@@ -50,6 +54,9 @@ int	xmlParserInputBufferRead		(xmlParserInputBufferPtr in,
 						 int len);
 int	xmlParserInputBufferGrow		(xmlParserInputBufferPtr in,
 						 int len);
+int	xmlParserInputBufferPush		(xmlParserInputBufferPtr in,
+						 int len,
+						 const char *buf);
 void	xmlFreeParserInputBuffer		(xmlParserInputBufferPtr in);
 char *	xmlParserGetDirectory			(const char *filename);
 
